//  MainWidget.cpp version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//
//

#define LINE_LENGTH     4096

#include <QUXKinput.h>
#include <stdio.h>
#include <stdlib.h>

#include "MainWidget.h"
#include "GoToDialog.h"
#include <Translate.h>
#include <string.h>
#include <strings.h>
#include <UFontFreeType.h>



QList<MainWidget> MainWidget::windowList;

MainWidget::MainWidget (const Config& configIn, 
	const char* fileNameIn,
	const char* encodingIn,
	const UWidget::UEditable editableIn) 
	: QObject (), config (configIn) 
{
	fileDialog = 0;
	saveDialog = 0;
	printDialog = 0;
	findDialog = 0;
	menuBar=0;
	menuBarAction=0;
	editArea = 0;
	fileName = 0;
	destroyTimer = 0;
	editableFlag = editableIn;

	if (encodingIn!=0)
	{
		config.setChecked (Config::ENCODING, encodingIn);
	}
	topLevelWidget = new MyTopLevelWidget(this);
	setupMainWidget(fileNameIn);
	topLevelWidget->resize (560, 400);
	topLevelWidget->show();
}

void
MainWidget::setupMainWidget(const char* fileNameIn)
{
	const char*  	inputVle;
	AString         fullFileName;
	

	encoding = (const char*) config.getChecked (Config::ENCODING);

#ifndef WITH_KDE
	layout = new QBoxLayout (topLevelWidget, QBoxLayout::TopToBottom);
#endif

	editArea = new QUWidget (topLevelWidget, (const char*)config.getChecked(Config::FONTNAME));
	editArea->setFontSize (atoi((const char*)config.getChecked(Config::FONTSIZE)));
	editArea->setFontWeight ((const char*)config.getChecked(Config::FONTWEIGHT));
	editArea->setFontSlant ((const char*)config.getChecked(Config::FONTSLANT));
	editArea->setFontSpacing ((const char*)config.getChecked(Config::FONTSPACING));
	editArea->setFontAddStyle ((const char*)config.getChecked(Config::FONT_ADD_STYLE));
	editArea->setFontAvgWidth ((const char*)config.getChecked(Config::FONT_AVG_WIDTH));
	menuBarAction = new KMenuBarAction (topLevelWidget);
	menuBar = new MenuBar (menuBarAction, &config);
	
	if (editArea->setConversion((const char*)config.getChecked(Config::ENCODING)) == UString::ERROR)
	{
		cerr << MAINPRG << ": can not find converter for encoding '" <<
			(const char*)config.getChecked(Config::ENCODING) <<
			"'\n";
	}
	inputVle = (const char*)config.getChecked(Config::INPUT);
	if (strcasecmp (inputVle, "Straight") ==0
		|| strcasecmp (inputVle, "None")==0)
	{
			editArea->setInput (0);
	}
	else if (editArea->setInput(inputVle) == UWidget::ERROR)
	{
		cerr << MAINPRG << ": can not set input '" <<
			(const char*)config.getChecked(Config::INPUT) << "'\n";
	}
	inputVle = (const char*)config.getChecked(Config::INPUT);
	if (inputVle!=0)
	{
		// set up xinput here when it is supported
	}
	
	editArea->setTabSize (
		atoi((const char*) config.getChecked (Config::TABSIZE)));
	editArea->setBackground (
		(const char*) config.getChecked (Config::WINDOW_BACKGROUND));
	editArea->setForeground(
		(const char*) config.getChecked (Config::WINDOW_FOREGROUND));
	editArea->setCursorColor(
		(const char*) config.getChecked (Config::CURSOR_COLOR));

	setXInput ((const char*) config.getChecked (Config::XINPUT));

#ifdef WITH_KDE
	dropZone = new KDNDDropZone (topLevelWidget, DndURL);

	connect (dropZone, SIGNAL (dropAction (KDNDDropZone *)),
		this, SLOT (dropEvent (KDNDDropZone *)));
#endif
	
	topLevelWidget->setCaption ((const char*) "Yudit");

	if (fileNameIn!=0)
	{
		editArea->setEditable (UWidget::EDITABLE);
		fullFileName = "file:";
		fullFileName+=fileNameIn;
		loadFile ((const char*) fullFileName);
		topLevelWidget->setCaption ((const char*) fullFileName);
	}
	editArea->setEditable (editableFlag);

	QObject::connect (menuBar, SIGNAL(changeConfigSignal( 
		Config::CheckList, const char*)), 
		this, SLOT (changeConfigSlot (Config::CheckList, const char*))); 
	QObject::connect (menuBar, SIGNAL(printSignal()), 
		this, SLOT (doPrintSlot()));

	QObject::connect (menuBar, SIGNAL (saveAsFile()), 
		this, SLOT (saveAs ()));

	QObject::connect (menuBar, SIGNAL (saveFile()), 
		this, SLOT (save ()));

	QObject::connect (menuBar, SIGNAL (loadFileSignal()), 
		this, SLOT (loadFile ()));

	QObject::connect (menuBar, SIGNAL (appendFileSignal()), 
		this, SLOT (appendFile ()));

	QObject::connect (menuBar, SIGNAL (reloadFileSignal()), 
		this, SLOT (reloadFile ()));

	QObject::connect (menuBar, SIGNAL(configSignal()), 
		this, SLOT (editConfig()));
	QObject::connect (menuBar, SIGNAL(quitSignal()), 
		this, SLOT (quitMainWidget()));
	QObject::connect (menuBar, SIGNAL(exitSignal()), 
		this, SLOT (exitMainWidget()));
	QObject::connect (menuBar, SIGNAL(newSignal()), 
		this, SLOT (newMainWidget()));
	QObject::connect (menuBar, SIGNAL(searchSignal(MenuBar::SearchType)), 
		this, SLOT (searchSlot(MenuBar::SearchType)));
	QObject::connect (menuBar, SIGNAL(helpSignal(MenuBar::HelpType)), 
		this, SLOT (helpSlot(MenuBar::HelpType)));

	// If you don't use show canvas won't be nicely sized in the first run
	
	menuBarAction->show();
#ifdef WITH_KDE
	topLevelWidget->setView (editArea);
	topLevelWidget->setMenu(menuBarAction);
#else
	layout->addWidget (editArea);
	layout->setMenuBar(menuBarAction);
#endif
	// Want a core?
	//layout->activate();

	windowList.append (this);

	editArea->refresh ();
}

MainWidget::~MainWidget ()
{
#ifndef WITH_KDE
	delete layout;
#endif
	if (destroyTimer!=0) delete destroyTimer;
	if (menuBar!=0) delete menuBar;
	if (menuBarAction!=0) delete menuBarAction;
	if (editArea!=0) delete editArea;
	if (fileDialog) delete fileDialog;
	if (saveDialog) delete saveDialog;
	if (printDialog) delete printDialog;
	if (topLevelWidget) delete topLevelWidget;
	windowList.remove(this);
}

void
MainWidget::changeConfigSlot (Config::CheckList type, const char* setTo)
{
	const char*	newVle;
	
	newVle = setTo;
	switch (type)
	{
	case Config::ENCODING:
		if (editArea->setConversion(newVle) == UString::ERROR)
		{
			QMessageBox::message ( Translate ("Yudit: Error"),
				Translate ("Can not set desired encoding."),
				Translate ("Ok"));
			return;
		}
		break;
	case Config::INPUT:
		if (strcasecmp (newVle, "Straight") ==0
			|| strcasecmp (newVle, "None")==0)
		{
			editArea->setInput (0);
		}
		else if (editArea->setInput(newVle) == UWidget::ERROR)
		{
			QMessageBox::message (
				Translate ("Yudit: Error"),
				Translate ("Can not set desired input map."),
				Translate ("Ok"));
			return;
		}
		break;
	case Config::XINPUT:
		if (setXInput (newVle)!=0)
		{
			QMessageBox::message (
				Translate ("Yudit: Error"),
				Translate ("Can not set up XInput."),
				Translate ("Ok"));
			return;
		}
		break;
	case Config::FONTNAME:
		if (editArea->setFontName (newVle) == UWidget::ERROR)
		{
			QMessageBox::message (
				Translate ("Yudit: Error"),
				Translate ("Can not set desired font."),
				Translate ("Ok"));
			return;
		}
		editArea->refresh();
		break;
	case Config::FONTWEIGHT:
		editArea->setFontWeight (newVle);
		editArea->refresh();
		break;
	case Config::FONT_AVG_WIDTH:
		editArea->setFontAvgWidth (newVle);
		editArea->refresh();
		break;
	case Config::FONT_ADD_STYLE:
		editArea->setFontAddStyle (newVle);
		editArea->refresh();
		break;
	case Config::FONTSIZE:
		editArea->setFontSize (atoi(newVle));
		editArea->refresh();
		break;
	case Config::FONTSLANT:
		editArea->setFontSlant (newVle);
		editArea->refresh();
		break;
	case Config::FONTSPACING:
		editArea->setFontSpacing (newVle);
		editArea->refresh();
		break;
	case Config::TABSIZE:
		editArea->setTabSize (atoi(newVle));
		break;
	case Config::WINDOW_BACKGROUND:
		editArea->setBackground (newVle);
		break;
	case Config::WINDOW_FOREGROUND:
		editArea->setForeground (newVle);
		break;
	case Config::CURSOR_COLOR:
		editArea->setCursorColor (newVle);
		break;
	default:
		return;
	}
	config.setChecked (type, newVle);
}

void
MainWidget::doPrintSlot ()
{
	PrintDialog::Button	printButton;
	const char*		gotErrors;

	if (printDialog == 0)
	{
		printDialog = new PrintDialog (topLevelWidget,
			&config, editArea);
	}
	printButton = printDialog->print(
			editArea->uGetTextBuffer(),
			editArea->uGetTextSizes(),
			editArea->uGetTextSize(),
			(const char*) fileName);
	if (printButton == PrintDialog::CANCEL)
	{
		return;
	}
	gotErrors = printDialog->getMessage();
	if (gotErrors==0)
	{
		QMessageBox::message (
			Translate ("Yudit: Warning"), 
			Translate ("Print failed.\n"
				"You might not have printing properly\n"
				"configured in your config file."),
			Translate ("Ok"));
		return;
	}
	QMessageBox::message (Translate ("Yudit: Info"), 
		gotErrors, Translate ("Ok"));
	
	return;
}

void
MainWidget::saveAs ()
{
	AString		fullFileName;
	int		status;

	if (saveDialog==0) saveDialog = new QFileDialog ();
	if (checkEncoding ()== 0) return;

	if (!writeDir.isNull())
	{
		saveDialog->setDir ((const char*) writeDir);
	}
	else if (!readDir.isNull())
	{
		saveDialog->setDir ((const char*) readDir);
	}
	const QString file = saveDialog->getSaveFileName();
	if (saveDialog->dirPath()!=0)
	{
		writeDir = saveDialog->dirPath();
	}
	if (!file.isNull())
	{
		fullFileName = "file:";
		fullFileName += (const char*) file;
		status=saveFile ((const char*) fullFileName);
		fileName= (const char*) fullFileName;
		topLevelWidget->setCaption ((const char*) fileName);
		encoding = (const char*) config.getChecked (Config::ENCODING);
		editableFlag=UWidget::EDITABLE;
		editArea->setEditable (editableFlag);
	}
}

void
MainWidget::save ()
{
	AString		errorMessage;
	const char*	gotErrors;

	if (editableFlag != UWidget::EDITABLE)
	{
		QMessageBox::message (
			Translate ("Yudit: Warning"), 
			Translate ("Buffer is not editable.\nTry 'Save As...'."),
			Translate ("Ok"));
		return;
	}
	if (fileName.isNull())
	{
		QMessageBox::message (
			Translate ("Yudit: Warning"), 
			Translate ("Buffer has no name.\nTry 'Save As...'."),
			Translate ("Ok"));
		return;
	}
	if (checkEncoding ()== 0) return;
	saveFile ((const char*) fileName);
	if (strncmp ((const char*) fileName, "file:",
		strlen("file:"))==0
		&& strcmp (&fileName[strlen("file:")],
		config.getConfigPath()) == 0)
	{
		gotErrors=config.testConfig ("yuditrc");
		if (gotErrors==0) return;
		errorMessage= (const char*) Translate (
			"The yuditrc config file you\n"
			"have just updated has errors.\n"
			"Do not quit until you fix them.\n\n");
		errorMessage+=gotErrors;
		QMessageBox::message (
			Translate ("Yudit: Warning"), 
			(const char*) errorMessage,
			Translate ("Ok"));
	}
	encoding = (const char*) config.getChecked (Config::ENCODING);
	return;
}

void
MainWidget::loadFile ()
{
	AString			fullFileName;
	
	if (editArea->isModified ())
	{
		if (!QMessageBox::query(
			Translate ("Yudit: Warning"),
			Translate ("Buffer has been modified.\nAbandon changes?"),
			Translate ("Yes"),
			Translate ("No"))) return;
	}

	if (fileDialog==0) fileDialog = new QFileDialog ();

	if (!readDir.isNull())
	{
		fileDialog->setDir ((const char*) readDir);
	}
	const QString file = fileDialog->getOpenFileName();
	if (fileDialog->dirPath()!=0)
	{
		readDir = fileDialog->dirPath();
	}
	
	if (!file.isNull())
	{
		fullFileName = "file:";
		fullFileName += (const char*) file;
		editableFlag=UWidget::EDITABLE;
		editArea->setEditable (editableFlag);
		loadFile ((const char*) fullFileName);
	}
	encoding = (const char*) config.getChecked (Config::ENCODING);
}

void
MainWidget::reloadFile ()
{
	AString			fullFileName;

	if (fileName.isNull())
	{
		QMessageBox::message(
			Translate ("Yudit: Warning"),
			Translate ("Buffer has no name.\nTry 'Save As...' first."),
			Translate ("Ok"));
		return;
	
	}
	if (editArea->isModified ())
	{
		if (!QMessageBox::query(
			Translate ("Yudit: Warning"),
			Translate ("Buffer has been modified.\nAbandon changes?"),
			Translate ("Yes"),
			Translate ("No"))) return;
	}
	if (checkEncoding ()== 0) return;
	if (!fileName.isNull())
	{
		fullFileName = (const char*) fileName; 
		editArea->setEditable (UWidget::EDITABLE);
		loadFile ((const char*) fullFileName);
	}
	editArea->setEditable (editableFlag);
	encoding = (const char*) config.getChecked (Config::ENCODING);
}

void
MainWidget::appendFile ()
{
	AString		fullFileName;

	if (editableFlag != UWidget::EDITABLE)
	{
		QMessageBox::message (
			Translate ("Yudit: Warning"), 
			Translate ("Buffer is not editable.\nTry 'Load'."),
			Translate ("Ok"));
		return;
	}
	if (fileDialog==0) fileDialog = new QFileDialog ();

	if (!readDir.isNull())
	{
		fileDialog->setDir ((const char*) readDir);
	}
	const QString file = fileDialog->getOpenFileName();
	if (fileDialog->dirPath()!=0)
	{
		readDir = fileDialog->dirPath();
	}
	if (!file.isNull())
	{
		fullFileName = "file:";
		fullFileName += (const char*) file;
		appendFile ((const char*) fullFileName);
	}
}

void
MainWidget::exitMainWidget ()
{
	if (!QMessageBox::query(
		Translate ("Yudit: Warning"),
		Translate ("Do you really want to exit all windows?"),
		Translate ("Yes"),
		Translate ("No")))
	{
		return;
	}
#ifdef WITH_KDE
	kapp->quit();
#endif
	exit (0);
}

void
MainWidget::editConfig ()
{
	(void) new MainWidget (this->getConfig(), config.getConfigPath(), "UTF8", 
		UWidget::EDITABLE);
}

void
MainWidget::quitMainWidget ()
{
	if (windowList.getFirst() == windowList.getLast())
	{
		if (editArea->isModified ())
		{
			if (!QMessageBox::query(
				Translate ("Yudit: Warning"),
				Translate ("Buffer has been modified.\nDo you really want to quit?"),
				Translate ("Yes"),
				Translate ("No"))) return;
		}
		else if (!QMessageBox::query(
			Translate ("Yudit: Warning"),
			Translate ("Do you really want to quit?"),
			Translate ("Yes"),
			Translate ("No")))
		{
			return;
		}
		// delete this;
#ifdef WITH_KDE
		kapp->quit();
#endif
		exit (0);
	}
	else if (editArea->isModified ())
	{
		if (!QMessageBox::query(
			Translate ("Yudit: Warning"),
			Translate ("Buffer has been modified.\nDo you really want to quit?"),
			Translate ("Yes"),
			Translate ("No"))) return;
	}

	// This dumps on solaris
	//delete this;

	destroyTimer = new QTimer (this);
	connect (destroyTimer, SIGNAL (timeout()), this, SLOT (destroySlot()));
	// 20 msec
	destroyTimer->start (20, TRUE);
}

void
MainWidget::destroySlot()
{
	delete this;
	return;
}

void
MainWidget::newMainWidget ()
{
	(void) new MainWidget(this->getConfig());
}

//
// Override MyTopLevelWidget
//
void
MainWidget::closeEvent (QCloseEvent *e)
{
	quitMainWidget ();
}

void
MainWidget::searchSlot (MenuBar::SearchType type)
{
	GoToDialog*	gotoDialog;
	int		lineNo;
	switch (type)
	{
	case MenuBar::FIND:
		if (findDialog ==0)
		{
			findDialog = new FindDialog (topLevelWidget, 
				editArea->getWidget(), &config);
		}
		findDialog->findString ();
		delete findDialog;
		findDialog = 0;
		break;
	case MenuBar::GOTO:
		gotoDialog = new GoToDialog (topLevelWidget, "Yudit: Go To", 
				Translate ("Line number:"));
		lineNo = gotoDialog->getLineNo();
		delete gotoDialog;
		if (lineNo==-1) return;
		editArea->uGoTo(lineNo);
		break;
	default:
		break;
	}
}

void
MainWidget::helpSlot (MenuBar::HelpType type)
{
	char*		filePathName;
	switch (type)
	{
	case MenuBar::ABOUT:
		QMessageBox::message (
			Translate ("Yudit Information"), 
			UWidgetVersion(),
			Translate ("Ok"));
		return;
	case MenuBar::GENERAL:
		filePathName = UFindFile (UGetMapPath (), "General.help");
		break;
	case MenuBar::CONFIG:
		filePathName = UFindFile (UGetMapPath (), "Config.help");
		break;
	case MenuBar::TESTPAGE:
		filePathName = UFindFile (UGetMapPath (), "TestPage.help");
		break;
	default:
		return;
	}
	if (filePathName==0)
	{
		QMessageBox::message (
			Translate ("Warning"), 
			Translate ("Can not find help file."),
			Translate ("Ok"));
		return;
	}
	(void) new MainWidget (this->getConfig(), filePathName, "UTF8", UWidget::VIEW);
	delete filePathName;
	return;
}

KMenuBarAction::KMenuBarAction (QWidget* parent) : KMenuBar (parent)
{
}

KMenuBarAction::~KMenuBarAction()
{
}

int
KMenuBarAction::aInsert (const char* text, QPopupMenu* menu)
{
	return insertItem (text, menu);
}

int
MainWidget::checkEncoding ()
{
	char		buffer[255];
	char		format[255];
	const char*	enc=(const char*) config.getChecked(Config::ENCODING);

	if (fileName.isNull() || enc==0 || encoding.isNull())
	{
		return 1;
	}
	if (strcmp ((const char*) encoding, enc)==0)
	{
		return 1;
	}
	strcpy (format, Translate (
		"The last file transaction on this buffer used encoding '%s'.\n"
		"Now your encoding is '%s'.\n\nShall we proceed?"));
	sprintf (buffer, format, (const char*) encoding, enc);
	if (!QMessageBox::query(
		Translate ("Warning"),
		buffer,
		Translate ("Yes"),
		Translate ("No")))
	{
		return 0;
	}
	return 1;
}

int
MainWidget::saveFile (const char* filenameIn)
{
	unsigned char*		text;
	int			textlen;
	ofstream*		file;
	int			error;

	if (strlen(filenameIn) > strlen ("file:") 
		&& strncmp (filenameIn, "file:", strlen ("file:")) == 0)
	{
		file = new ofstream (filenameIn + strlen ("file:"));
	}
	else
	{
		file = new ofstream (filenameIn);
	}
	if (file->rdbuf() == 0 || !file->rdbuf()->is_open())
	{
		QMessageBox::message (
			Translate ("Error"), 
			Translate ("Write failed."),
			Translate ("Ok"));
		delete file;
		return -1;
	}
	error=0;
	text = editArea->getBuffer(&textlen);
	if (text !=0)
	{
		if (UWriteBuffer(file, text, textlen) ==-1) error=1;
		delete text;
	}
	file->close();
	delete file;
	if (error)
	{
		QMessageBox::message (
			Translate ("Error"), 
			Translate ("Write failed."),
			Translate ("Ok"));
		return -1;
	}
	editArea->clearModified ();
	return 1;
}

int
MainWidget::loadFile (const char* filenameIn)
{
	int	retVle;
	editArea->clearText();
	retVle =  appendFile (filenameIn);
	editArea->clearModified ();
	if (retVle>=0)
	{
		fileName = (const char*) filenameIn;
		encoding = (const char*) config.getChecked (Config::ENCODING);
		topLevelWidget->setCaption ((const char*) fileName);
	}
	else
	{
		fileName = (const char*) 0;
		encoding = (const char*) 0;
		topLevelWidget->setCaption ((const char*) "Yudit");
	}
	return retVle;
}


int
MainWidget::appendFile (const char* filenameIn)
{
	int	ret_vle;
	ret_vle = appendFileBase (filenameIn);
	editArea->refresh();
	if (ret_vle>=0)
	{
		if (fileName.isNull ()) 
		{
			fileName = (const char*) filenameIn;
			encoding = (const char*) config.getChecked (Config::ENCODING);
			topLevelWidget->setCaption ((const char*) fileName);
		}
	}
	return ret_vle;
}

int
MainWidget::appendFileBase (const char* filenameIn)
{
	ifstream*	file;
	unsigned char*  buffer;
	int		buflen;

	if (strlen(filenameIn) > strlen ("file:") 
		&& strncmp (filenameIn, "file:", strlen ("file:")) == 0)
	{
		file = new ifstream (filenameIn + strlen ("file:"));
	}
	else
	{
		file = new ifstream ((const char*) filenameIn);
	}
	if (file->rdbuf() == 0 || !file->rdbuf()->is_open())
	{
		cerr << MAINPRG << ": can not open '" << 
			filenameIn << "' to read.";
		QMessageBox::message (
			Translate ("Error"), 
			Translate ("Can not read file."),
			Translate ("Ok"));
		delete file;
		return -1;
	}
	buffer = UReadBuffer (file, &buflen);
	editArea->appendText (buffer, buflen);
	delete buffer;
	file->close();
	delete file;
	return 1;
}

#ifdef WITH_KDE
void
MainWidget::dropEvent (KDNDDropZone *dropZone_)
{
	QStrList &list = dropZone_->getURLList();
	char*		str;
	int		ret_vle;
	int		i;

	// It would be really kool to make it work with url
	i=0;
	fileName=(const char*) 0;
	encoding=(const char*) 0;
	for (str = list.first(); str != 0; str = list.next())
	{
		editArea->clearText();
		ret_vle = appendFileBase (str);
		editArea->refresh();
		if (ret_vle>=0 && i==0)
		{
			fileName = (const char*) str;
			encoding = (const char*) config.getChecked (Config::ENCODING);
			topLevelWidget->setCaption ((const char*) fileName);
		}
		i++;
	}
	if (fileName.isNull ()) topLevelWidget->setCaption ("Yudit");

	editArea->refresh ();
	
}

#else
void
MainWidget::dropEvent (KDNDDropZone *dropZone_)
{
}
#endif

MyTopLevelWidget::MyTopLevelWidget (QObject *parentIn) : KTopLevelWidget()
{
	parent = parentIn;
}

MyTopLevelWidget::~MyTopLevelWidget()
{
}

void
MyTopLevelWidget::closeEvent (QCloseEvent *e)
{
	((MainWidget*)parent)->closeEvent (e);
}

int
MainWidget::setXInput (const char* nameIn)
{
	const XInputMap*	xInputMap;

	xInputMap = config.getXInputMap (nameIn);
	if (xInputMap==0)
	{
		cerr << "error: can not get XInputMap.\n";
		return -1;
	}
	return editArea->uSetXInput (nameIn, 
			(const char*) xInputMap->list[XInputMap::TYPE],
			(const char*) xInputMap->list[XInputMap::STYLE],
			(const char*) xInputMap->list[XInputMap::TEXT_TYPE],
			(const char*) xInputMap->list[XInputMap::ENCODING],
			(const char*) xInputMap->list[XInputMap::FONTMAP],
			atof ((const char*)xInputMap->list[XInputMap::VERSION]));
}
